******************************************************************************************************
** PROJECT     : QUOTA                                                                              **
** TITLE       : QUOTA_GUIDE                                                                         *
**             :                                                                                     *
** VERSION     : 1.00                                                                                *
** DATE        : 20071112                                                                            *
** AUTHOR      : DGS                                                                                 *
** HISTORY     : VERSION 1.00 -- Taken from quota_guide_Sequencer_V001x                              *
**             :                 Added binning                                                       *
**             :                 Added fast read routine for guide                                   *
**             :                 Added serial prescan                                                *
**             :                                                                                     *
** DESCRIPTION : Designed to support QUOTA readout and guide / shift functionality                   *
**             : This code must be run with MCB & CCDAcq board firmware version 4.95                 *
**             :                                                                                    **
******************************************************************************************************
** STA OTA Configuration (Lifted from sta_sequencerV010.asm)                                        **
*                                                                                                    *
*                                                                                                    *
*                            P->   4 22   4 22   4 22                                                *
*                            P->   433    433    433                                                 *
*                            P->    111    111    111                                                *
*                                -----------------------                                             *
*       OTG|SW-2-1|S3-2-1|S3-2-1|S3-2-1|S3-2-1|S3-2-1....                                            *
*        |   <-3 prescan pixels->    parallels dump into S2 and S1                                   *
*        |    OD                                                                                     *
*       _|_   _|                                                                                     *
*     _|   |_|                                                                                       *
* RG-|_      |_                                                                                      *
*      |       |                                                                                     *
*     RD      OS                                                                                     *
*                                                                                                    *
*  NOTE: PSL (standby low) is connected to P3 and P4                                                 *
*        PSH (standby high) is connected to P1 and P2                                                *
*                                                                                                    *
* During integration P1 and P2 are high, so S1 must be low to prevent leakage from the parallels     *
* into the serials.  As part of the transition from the integration state to a serial transfer, S1   * 
* must be set high.  Then S1 will be set low as the first serial charge transfer. Integrate on reset *
* will occur when S2 is high, before S3 (and SW) are set high.                                       *
**                                                                                                  **
******************************************************************************************************

*************************
**  CONSTANT DEFINITIONS
*************************

** CCD BOARD ROW / COLUMN SELECT MODE SELECT KEYS
DEF #DISABLE_ALL x0001FFFF
DEF #STANDBY_ALL x0002FFFF
DEF #ACTIVE_ALL  x0003FFFF
DEF #ENABLE_R0   x000401FF          * video is enabled for OTA row 0
DEF #ENABLE_R1   x000402FF          * video is enabled for OTA row 1
DEF #ENABLE_R2   x000404FF          * video is enabled for OTA row 2
DEF #ENABLE_R3   x000408FF          * video is enabled for OTA row 3
DEF #ENABLE_R4   x000410FF          * video is enabled for OTA row 4
DEF #ENABLE_R5   x000420FF          * video is enabled for OTA row 5
DEF #ENABLE_R6   x000440FF          * video is enabled for OTA row 6
DEF #ENABLE_R7   x000480FF          * video is enabled for OTA row 7
DEF #ACTIVE_GUIDE x0005FFFF         * Activate just the guide regions designated by the guide mask registers (no video)
DEF #ENABLE_GUIDE x0006FFFF         * Enable just the guide regions designated by the guide mask registers (with video)

** CLOCK BOARD FIRMWARE SHIFT PATTERNS
DEF #CLEAR_SHIFT xFE000000           * -512 vertical shifts i.e. 512 shifts towards serial register
DEF #IDLE_SHIFT  xFFF80000	     * -8 veritcal shifts toward serial register
DEF #READ_SHIFT  xffff0000           * -1 vertical shift to transfer charge to serial register

**********************
** TIMING CONSTANTS ** 
**********************
DEF #RST_WIDTH       13              * Delay for Reset on time    (clk) 13*50 ns + 3 instructions = 800ns
DEF #RST_SETTLE      13              * Delay for signal to settle (clk) 13*50 ns + 3 instructions = 800ns
DEF #SER_OVERLAP     10              * Delay serial clock overlap (clk) 10*50 ns = 500ns
DEF #GDE_OVERLAP      6		     * Delay serial clock overlap during guide readout (clk) 6*50 ns=300ns
DEF #CLR_OVERLAP      6              * Used to shift serials during clear operation (clk) = 300 ns
DEF #SCI_DWELL       20              * Entered in clk cycles 20*50 ns + 2 instructions = 1100ns
DEF #GDE_DWELL       10              * Entered in clk cycles 10*50 ns + 2 instructions = 600ns

*******************************
** LOOP REGISTER DEFINITIONS **
*******************************
DEF #ROWCNT          0              * NUMBER OF BINNED ROWS TO READOUT (SciRowPixels/RowBin)
DEF #COLCNT          1              * NUMBER OF BINNED COLUMNS TO READOUT (SciColPixels/ColBin)
DEF #ROWBIN          2              * BINNING FACTOR FOR ROWS
DEF #COLBIN          3              * BINNING FACTOR FOR COLS
DEF #GDECNT          4              * NUMBER OF BINNED ROWS AND COLUMNS IN GUIDE BOX
DEF #GDEROWBIN       5              * Binning factor for GdeRows
DEF #GDECOLBIN       6              * Binning factor for GdeCols (GdeRows-1)
DEF #SPARE_7         7              * not used
DEF #SPARE_8         8              * not used
DEF #SPARE_9         9              * not used
DEF #SPARE_10       10              * not used
DEF #SPARE_11       11              * not used
DEF #SPARE_12       12              * not used
DEF #SPARE_13       13              * not used
DEF #SPARE_14       14              * not used
DEF #SPARE_15       15              * not used

***********************************************
** PATTERN POINTER LOOP REGISTER DEFINITIONS **
***********************************************
DEF #CLEAR_ROWS      64             * Number of row transfers to loop through (#CLEAR_ROWS * #CLEAR_BINNING = Effective number of rows cleared) 
DEF #CLEAR_BINNING    8             * Row binning factor for clear array operation (IDLE_SHIFT used instead of this to reduce logic activity)
DEF #CLEAR_COLS     512             * Number of columns to shift for each binned row transfer
DEF #PRESCAN	      3		    * Number of prescan pixels

**********************************
**    JUMP FLAG CONSTANTS       **
** Bit position in CMD Register **
**********************************
DEF #GD_ITC       1                 * Jump on Guide Integration time expired
DEF #SC_ITC       2                 * Jump on Science Integration time expired
DEF #START        3                 * Jump on start exposure flag set
DEF #EXPOSE       4                 * Begin / Maintain Science exposure routine (Start vector bit 0) 
DEF #GUIDE        5                 * Begin / Maintain Guide exposure routine (Start vector bit 1)
DEF #SHIFT        6                 * Begin shift process (Start vector bit 2)
DEF #FOCUS        7                 * Begin focus frame
DEF #TESTBIT      8                 * Spare
DEF #RUN_EXPOSE   9                 * Local flag to run science exposure (Under sequencer control)
DEF #RUN_GUIDE   10                 * Local flag to run guide exposure loop (Under sequencer control)
DEF #SYNC        11                 * Jump on SYNC bit for multiple DHE environment  (Under sequencer control)
DEF #CONTRUN     12                 * Sequencer code continous run enable bit (Usr0Flag)
DEF #GBIN_N      13                 * Sequencer function flag bit 13
DEF #SBIN_N      14                 * Sequencer function flag bit 14
DEF #FUNC_BUSY   15                 * Sequencer functions 1=Busy 0=Done, use this bit to monitor
                                    * shift logic operations on CLK or CCD boards 
                                    * and mode setting on CCD board.

****************************************
** Constants and Assignments
****************************************
** EFR Reg constants
DEF #KILL_EXPFLG    x00000001       * Cancel start exposure flag using the EFR register
DEF #STRT_SCI_ITC   x00000002       * Start the Science integration time counter using the EFR register
DEF #STOP_SCI_ITC   x00000004       * Stop the Science integration time counter using the EFR register
DEF #STRT_GD_ITC    x00000008       * Start the Guide integration time counter using the EFR register
DEF #STOP_GD_ITC    x00000010       * Stop the Guide integration time counter using the EFR register
DEF #DISABLE_LSR    x00000100       * Disables automatic board select cancel after an LSR instruction
DEF #ENABLE_LSR     x00000200       * Enables automatic board select cancel after an LSR instruction.
DEF #SET_SYNC_LOW   x00004000       * If enabled as master DHE, set sync bit low after sync delay
DEF #SET_SYNC_HIGH  x00008000       * If enabled as master DHE, set sync signal high immediately
DEF #RESET_BIT_4    x00010000       * Reset bit 4 of the sequencer command register.
DEF #SET_BIT_4      x00020000       * Set bit 4 of the sequencer command register.
DEF #RESET_BIT_5    x00040000       * Reset bit 5 of the sequencer command register.
DEF #SET_BIT_5      x00080000       * Set bit 5 of the sequencer command register.
DEF #RESET_BIT_6    x00100000       * Reset bit 6 of the sequencer command register.
DEF #SET_BIT_6      x00200000       * Set bit 6 of the sequencer command register.
DEF #RESET_BIT_7    x00400000       * Reset bit 7 of the sequencer command register.
DEF #SET_BIT_7      x00800000       * Set bit 7 of the sequencer command register.
DEF #RESET_TESTBIT  x01000000       * Reset bit 8 of the sequencer command register.
DEF #SET_TESTBIT    x02000000       * Set bit 8 of the sequencer command register. -- Used to flag external signal to Oscilloscope
DEF #RESET_BIT_9    x04000000       * Reset bit 9 of the sequencer command register.
DEF #SET_BIT_9      x08000000       * Set bit 9 of the sequencer command register.
DEF #RESET_BIT_10   x10000000       * Reset bit 10 of the sequencer command register.
DEF #SET_BIT_10     x20000000       * Set bit 10 of the sequencer command register.
DEF #KILLANDSYNC    x00004001       * Set sync low and kill start exposure flag

** Mode register constants
DEF #RESET       0                  * Mode register constant for board reset operation
DEF #READ        1                  * Mode register constant for read operation
DEF #WRITE16     2                  * Mode register constant for 16 bit write operation
DEF #WRITE32     3                  * Mode register constant for 32 bit write operation

** LSR Board select assignments
DEF #DSLCT    x00
DEF #MCB      x01                   * DHE Slot 1
DEF #CCDCLK   x02                   * DHE Slot 2
DEF #CCDACQ_1 x04                   * DHE Slot 3
DEF #CCDACQ_2 x08                   * DHE Slot 4
DEF #CCDACQ_3 x10                   * DHE Slot 5
DEF #CCDACQ_4 x20                   * DHE Slot 6
DEF #CCDACQ_ALL x3C                 * DHE Slots 3,4,5,6.

** CCDCLK BOARD ADDRESS POINTERS
DEF #CLKPORT     x0000
DEF #FBIAS0123   x0001
DEF #FBIAS4567   x0002

** CCDACQ BOARD ADDRESS POINTERS
DEF #CDSREG         x0000           * Write raw 32-bit data to the CDS control register
DEF #DOPREG         x0001           * Write raw 32-bit data to the DOP port
DEF #GUIDE_ACTIVE   x0002           * Write a 16-bit mask value to set the referenced guide cells to enabled without video - see note 3. 
DEF #SCIENCE_ACTIVE x0003           * Write a 16-bit mask value to set the referenced science cells to active without video - See note 4. 
DEF #BRST_CMD       x0004
DEF #SEQ_CMD        x0005           * Trigger the Acquisition board micro sequencer - See note 1.
DEF #MODE_CMD       x0006           * OTA Mode control register - Controls the OTA cell states as disabled/enabled/active - See note 2.
DEF #SHIFT_CMD      x0007           * Write a 32-bit word to define an x/y shift for all ot cells in the active state - See note 5.

DEF #MODE_ACTIVE    x0007           * Set cells active in mask word written to this address
DEF #TRIG_SHIFT     x1040           * Write to this register to initaiate shift list structure processing
                                    * write a '0' for guide shift list or '1' for science shift list processing  
DEF #PROC_GUIDE     x10400000       * BUT since this is a 16 bit write mode command, we prepend the address
DEF #PROC_SCIENCE   x10400001

********************** CCD Acquisition Firmware Function code Details ***********************************
** Notes                                                                                               **
**    1     Writing to this register triggers the CDS process micro sequencer. The 32-bit word written **
**          to this register controls the channel number to use, the start origin of the sequencer code**
**          and the run length. The formate of the word is:                                            **
**          ----------------------------------------------------------------------------               **
**          | BITS 31 DOWNTO 24 | BITS 23 DOWNTO 16 | BITS 15 DOWNTO 8| BITS 7 DOWNTO 0|               **
**          |-------------------|-------------------|-----------------|----------------|               **
**          |     NOT USED      | CDS CHANNEL MASK  |    LENGTH OF    | STARTING ADDR  |               **
**          |                   |                   |    SEQUENCE     | OF SEQ IN RAM  |               **
**          ----------------------------------------------------------------------------               **
**                                                                                                     **
**    2     THIS FUNCTION CONTROLS THE MODE OF OTA CELLS BY USING THE ROW AND COLUMN                   **
**          SELECT AND D0,D1,D2 SIGNALS ASSIGNED TO THE DOP PORT BITS.                                 **
**          THERE ARE 4 MODES AVAILABLE THAT ARE SET BY WRITING A CODE TO THE MODE                     **
**          REGISTER ALONG WITH A ROW AND COLUMN MASK PATTERN. THE FORMAT FOR THE                      **
**          MODE WORD IS:                                                                              **
**          ----------------------------------------------------------------                           **
**          |    31:19    |    18:16   |     15:8      |        7:0       |                            **
**          |   NOT USED  | MODE  BITS | ROW MASK BITS | COLUMN MASK BITS |                            **
**          ----------------------------------------------------------------                           **
**          MODE  MODE CONSTANT DESCRIPTION                                                            **
**           0    MOD_NOP       DO NOTHING                                                             **
**           1    MOD_DISABLE   USE THE ROW AND COL MASKS TO SET THE CORRESPONDING                     **
**                              CELLS TO DISABLE MODE.                                                 **
**           2    MOD_STANDBY   USE THE ROW AND COL MASKS TO SET THE CORRESPONDING                     **
**                              CELLS TO STANDBY MODE (USED DURING INTEGRATION)                        **
**           3    MOD_ACTIVE    USE THE ROW AND COL MASKS TO SET THE CORRESPONDING                     **
**                              NON-GUIDE CELLS TO ACTIVE WITH VIDEO DISABLED (USED FOR SHIFTING)      **
**           4    MOD_ENABLED   USE THE ROW AND COL MASKS TO SET THE CORRESPONDING                     **
**                              NON-GUIDE CELLS TO ACTIVE WITH VIDEO ENABLED  (USED FOR READOUT)       **
**           5    MOD_GDACTIVE  USE THE ROW AND COL MASKS TO SET THE CORRESPONDING                     **
**                              GUIDE CELLS TO ACTIVE WITH VIDEO DISABLED (USED FOR SHIFTING)          **
**           6    MOD_GDENABLE  USE THE ROW AND COL MASKS TO SET THE CORRESPONDING                     **
**                              GUIDE CELLS TO ACTIVE WITH VIDEO ENABLED  (USED FOR GUIDE READOUT)     **
**                                                                                                     **
**    3     Writing a 16-bit mask word to this register is equivelent to writing a 32-bit word to the  **
**          MODE_CMD register with the mode set to MOD_GDACTIVE. This just enables the sequencer to    **
**          use this mode on masks dynamically stored in the pattern data segment without having to    **
**          prepend the mode code.                                                                     **
**                                                                                                     **
**    4     Writing a 16-bit mask word to this register is equivelent to writing a 32-bit word to the  **
**          MODE_CMD register with the mode set to MOD_ACTIVE. This just enables the sequencer to      **
**          use this mode on masks dynamically stored in the pattern data segment without having to    **
**          prepend the mode code.                                                                     **
**                                                                                                     **
**    5     Write a 32-bit word to move charge on any active OTA cell by x/y amount between -32767 and **
**          32766 cells (a little over the top but you never know what tomorrows detectors may look    **
**          like !). The format of the 32-bit word is:                                                 **
**          ----------------------------------------------------------------------------               **
**          |                 31:16              |                  15:0               |               **
**          |  SIGNED ROW SHIFT VALUE (Y SHIFT)  | SIGNED COLUMN SHIFT VALUE (X SHIFT) |               **
**          ----------------------------------------------------------------------------               **
**           BITS 31:16  TWOS COMPLIMENT VALUE FOR ROW SHIFT (Y-SHIFT).                                **
**           BITS  15:0  TWOS COMPLIMENT VALUE FOR COLUMN SHIFT (X-SHIFT).                             **
**           THE INITIAL AND FINAL CLOCK PATTERN PRESENT ON THE PARALLELS IS                           **
**               P1 = LOW   = DOP BIT 19                                                               **
**               P2 = HIGH  = DOP BIT 20                                                               **
**               P3 = HIGH  = DOP BIT 21                                                               **
**               P4 = LOW   = DOP BIT 22                                                               **
**           THE SELECTED CELLS ARE Y-SHIFTED FIRST, THEN X-SHIFTED.                                   **
**                                                                                                     **
*********************************************************************************************************
                                   
*********************************************
** CLOCK SIGNAL DEFINITIONS                **
*********************************************
DEF #Ota02_S3   b10000000000000000000000000000000    * Coupled to CLK_OUT_31
DEF #SHUTTER    b01000000000000000000000000000000    * Coupled to CLK_OUT_30
DEF #Ota01_RG   b00100000000000000000000000000000    * Coupled to CLK_OUT_29
DEF #CLK28      b00010000000000000000000000000000    * Coupled to CLK_OUT_28
DEF #CLK27      b00001000000000000000000000000000    * Coupled to CLK_OUT_27
DEF #Ota03_RG   b00000100000000000000000000000000    * Coupled to CLK_OUT_26
DEF #CLK25      b00000010000000000000000000000000    * Coupled to CLK_OUT_25
DEF #CLK24      b00000001000000000000000000000000    * Coupled to CLK_OUT_24
DEF #Ota04_RG   b00000000100000000000000000000000    * Coupled to CLK_OUT_23
DEF #CLK22      b00000000010000000000000000000000    * Coupled to CLK_OUT_22
DEF #Ota02_SW   b00000000001000000000000000000000    * Coupled to CLK_OUT_21
DEF #Ota02_S1   b00000000000100000000000000000000    * Coupled to CLK_OUT_20
DEF #CLK18      b00000000000010000000000000000000    * Coupled to CLK_OUT_19
DEF #CLK17      b00000000000001000000000000000000    * Coupled to CLK_OUT_18
DEF #Ota01_S1   b00000000000000100000000000000000    * Coupled to CLK_OUT_17
DEF #CLK16      b00000000000000010000000000000000    * Coupled to CLK_OUT_16
DEF #CLK15      b00000000000000001000000000000000    * Coupled to CLK_OUT_15
DEF #Ota03_S1   b00000000000000000100000000000000    * Coupled to CLK_OUT_14
DEF #CLK13      b00000000000000000010000000000000    * Coupled to CLK_OUT_13
DEF #Ota04_S1   b00000000000000000001000000000000    * Coupled to CLK_OUT_12
DEF #CLK11      b00000000000000000000100000000000    * Coupled to CLK_OUT_11
DEF #Ota02_RG   b00000000000000000000010000000000    * Coupled to CLK_OUT_10
DEF #Ota02_S2   b00000000000000000000001000000000    * Coupled to CLK_OUT_09
DEF #Ota01_SW   b00000000000000000000000100000000    * Coupled to CLK_OUT_08
DEF #Ota01_S3   b00000000000000000000000010000000    * Coupled to CLK_OUT_07
DEF #Ota01_S2   b00000000000000000000000001000000    * Coupled to CLK_OUT_06
DEF #Ota03_SW   b00000000000000000000000000100000    * Coupled to CLK_OUT_05
DEF #Ota03_S3   b00000000000000000000000000010000    * Coupled to CLK_OUT_04
DEF #Ota03_S2   b00000000000000000000000000001000    * Coupled to CLK_OUT_03
DEF #Ota04_SW   b00000000000000000000000000000100    * Coupled to CLK_OUT_02
DEF #Ota04_S3   b00000000000000000000000000000010    * Coupled to CLK_OUT_01
DEF #Ota04_S2   b00000000000000000000000000000001    * Coupled to CLK_OUT_00

*********************************************
** SERIAL CLOCK WAVEFORM CONSTRUCTION      **
*********************************************
** COMBINE ALL OTA SERIAL CLOCK INTO SUB GROUPS
DEF #S1   #Ota01_S1 or #Ota02_S1 or #Ota03_S1 or #Ota04_S1
DEF #S2   #Ota01_S2 or #Ota02_S2 or #Ota03_S2 or #Ota04_S2
DEF #S3   #Ota01_S3 or #Ota02_S3 or #Ota03_S3 or #Ota04_S3
DEF #SW   #Ota01_SW or #Ota02_SW or #Ota03_SW or #Ota04_SW
DEF #RG   #Ota01_RG or #Ota02_RG or #Ota03_RG or #Ota04_RG

** SERIAL CLOCK BIT PATTERNS for S1-S2-S2 clocking (charged transferred to S1 and S2)
DEF #S_STATE_1     #S1                    * This is the state during serial RESET
DEF #S_STATE_12    #S1 OR #S2             * This is the state for parallel transfers
DEF #S_STATE_2     #S2                    * State for integration and OT shift
DEF #S_STATE_23W   #S2 OR #S3 OR #SW
DEF #S_STATE_3W    #S3 OR #SW
DEF #S_STATE_13W   #S1 OR #S3 OR #SW
DEF #S_STATE_1W    #S1 OR #SW

** SERIAL CLOCKING VECTORS
DEF #SER_0      #S_STATE_2 OR #RG
DEF #SER_1      #S_STATE_2
DEF #SER_2      #S_STATE_23W
DEF #SER_3      #S_STATE_3W
DEF #SER_4      #S_STATE_13W
DEF #SER_5	#S_STATE_1W		* State during RST INT
DEF #SER_6      #S_STATE_1 		* Video dump - state during VID INT
DEF #SER_7      #S_STATE_12		* This is the state for PXFER

DEF #CLR_1      #S_STATE_2 OR #RG
DEF #CLR_2      #S_STATE_23W OR #RG
DEF #CLR_3      #S_STATE_3W OR #RG
DEF #CLR_4      #S_STATE_13W OR #RG
DEF #CLR_5      #S_STATE_1 OR #RG 
DEF #CLR_6      #S_STATE_12 OR #RG

DEF #GDE_0      #SER_0 OR #SHUTTER
DEF #GDE_1      #SER_1 OR #SHUTTER
DEF #GDE_2      #SER_2 OR #SHUTTER
DEF #GDE_3      #SER_3 OR #SHUTTER
DEF #GDE_4      #SER_4 OR #SHUTTER
DEF #GDE_5      #SER_5 OR #SHUTTER 
DEF #GDE_6      #SER_6 OR #SHUTTER
DEF #GDE_7      #SER_7 OR #SHUTTER
 
DEF #GCLR_1      #CLR_1 OR #SHUTTER
DEF #GCLR_2      #CLR_2 OR #SHUTTER
DEF #GCLR_3      #CLR_3 OR #SHUTTER
DEF #GCLR_4      #CLR_4 OR #SHUTTER
DEF #GCLR_5      #CLR_5 OR #SHUTTER
DEF #GCLR_6      #CLR_6 OR #SHUTTER

** SHUTTER PULSE DEFINITIONS **
DEF #SHUT_OPEN  #SHUTTER OR #S2     * Clock 30 used as a shutter driver ... 
DEF #SHUT_CLOSE #S2

** CCD ACQ BOARD MICROSEQUENCER START VECTORS
DEF #CDSCMD0      x0010B00           * Science Readout microcode origin
DEF #CDSCMD1      x0010B10           * Guide Readout microcode origin

********************************
** PATTERN MEMORY DEFINITIONS **
********************************
** PATTERN MEMORY SEGMENTATION MAP
DEF #PATSEG_CONSTANTS           x0000 * Pattern memory origin - Constant Values
DEF #MODE_SLCT_SEG              x0050 * Mode command words here.
DEF #SERIAL_WAVEFORMS           x0060 * Serial clocking patterns
DEF #PARALLEL_WAVEFORMS         x0070 * Parallel clocking patterns
DEF #SHIFT_PATTERNS             x0080 * Clock board dedicated shift patterns
DEF #CDS_COMMAND_VECTORS        x0090 * CCD Acq board microsequencer command vectors
DEF #PATSEG_DR                  x00A0 * Drain on/off patterns
DEF #CLEAR_WAVEFORMS            x00D0 * Clear process waveforms
DEF #GUIDE_WAVEFORMS            x00F0 * Guide specific serial waveforms (with shutter open)

***************************
** DAC SETTING CONSTANTS ** 
***************************
DEF #DR_ONVALUE     3600        * Sets Drain to 22V
*DEF #DR_OFFVALUE   3600        * Sets Drain to 22V (drains never go off)
*DEF #DR_ONVALUE     3119        * Sets Drain to 19V
*DEF #DR_OFFVALUE   3119        * Sets Drain to 19V (drains never go off)
*DEF #DR_OFFVALUE   1641        * Sets Drain to 10V
DEF #DR_OFFVALUE    656         * Sets Drain to 4V
DEF #DR0_DACADDR    x02200000
DEF #DR1_DACADDR    x02210000
DEF #DR2_DACADDR    x02220000
DEF #DR3_DACADDR    x02230000
DEF #DR4_DACADDR    x02240000
DEF #DR5_DACADDR    x02250000
DEF #DR6_DACADDR    x02260000
DEF #DR7_DACADDR    x02270000
DEF #DR0_ONCODE     #DR0_DACADDR OR #DR_ONVALUE
DEF #DR0_OFFCODE    #DR0_DACADDR OR #DR_OFFVALUE
DEF #DR1_ONCODE     #DR1_DACADDR OR #DR_ONVALUE
DEF #DR1_OFFCODE    #DR1_DACADDR OR #DR_OFFVALUE
DEF #DR2_ONCODE     #DR2_DACADDR OR #DR_ONVALUE
DEF #DR2_OFFCODE    #DR2_DACADDR OR #DR_OFFVALUE
DEF #DR3_ONCODE     #DR3_DACADDR OR #DR_ONVALUE
DEF #DR3_OFFCODE    #DR3_DACADDR OR #DR_OFFVALUE
DEF #DR4_ONCODE     #DR4_DACADDR OR #DR_ONVALUE
DEF #DR4_OFFCODE    #DR4_DACADDR OR #DR_OFFVALUE
DEF #DR5_ONCODE     #DR5_DACADDR OR #DR_ONVALUE
DEF #DR5_OFFCODE    #DR5_DACADDR OR #DR_OFFVALUE
DEF #DR6_ONCODE     #DR6_DACADDR OR #DR_ONVALUE
DEF #DR6_OFFCODE    #DR6_DACADDR OR #DR_OFFVALUE
DEF #DR7_ONCODE     #DR7_DACADDR OR #DR_ONVALUE
DEF #DR7_OFFCODE    #DR7_DACADDR OR #DR_OFFVALUE

******************************
** SHIFT OVERLAP CONSTANTS  **
******************************
DEF #PCLKOVLP_ADDR	 x00500000
DEF #MODE_CLK_OVLP	x1e00	* sets the logic overlap to 3 us (value 30 x 100ns = 3 us)
DEF #SCI_OVLP_VALUE	60	* sets the parallel clock overlap to 5 us during science readout
DEF #GDE_OVLP_VALUE	30	* sets the parallel clock overlap to 3 us during guide readout
DEF #SHIFT_OVLP_VALUE	80	* sets the parallel clock overlap to 7 us during charge shifting
DEF #SCI_READ_OVLP	#PCLKOVLP_ADDR OR #MODE_CLK_OVLP OR #SCI_OVLP_VALUE
DEF #GDE_READ_OVLP	#PCLKOVLP_ADDR OR #MODE_CLK_OVLP OR #GDE_OVLP_VALUE
DEF #SHIFT_OVLP		#PCLKOVLP_ADDR OR #MODE_CLK_OVLP OR #SHIFT_OVLP_VALUE

*************************************
** PATTERN MEMORY DATA DEFINITIONS **
*************************************
ORG PAT #PATSEG_CONSTANTS              * GENERAL CONSTANTS KEPT IN PAT MEM
PAT #KILL_EXPFLG        #KILL_EXPFLG  
PAT #STRT_SCI_ITC       #STRT_SCI_ITC 
PAT #STOP_SCI_ITC       #STOP_SCI_ITC 
PAT #STRT_GD_ITC        #STRT_GD_ITC  
PAT #STOP_GD_ITC        #STOP_GD_ITC  
PAT #DISABLE_LSR        #DISABLE_LSR  
PAT #ENABLE_LSR         #ENABLE_LSR   
PAT #SET_SYNC_LOW       #SET_SYNC_LOW 
PAT #SET_SYNC_HIGH      #SET_SYNC_HIGH
PAT #RESET_EXPOSE       #RESET_BIT_4  
PAT #SET_EXPOSE         #SET_BIT_4    
PAT #RESET_GUIDE        #RESET_BIT_5  
PAT #SET_GUIDE          #SET_BIT_5    
PAT #RESET_SHIFT        #RESET_BIT_6  
PAT #SET_SHIFT          #SET_BIT_6    
PAT #RESET_FOCUS        #RESET_BIT_7  
PAT #SET_FOCUS          #SET_BIT_7    
PAT #RESET_TESTBIT       #RESET_TESTBIT  
PAT #SET_TESTBIT         #SET_TESTBIT    
PAT #RESET_RUN_EXPOSE   #RESET_BIT_9  
PAT #SET_RUN_EXPOSE     #SET_BIT_9    
PAT #RESET_RUN_GUIDE    #RESET_BIT_10 
PAT #SET_RUN_GUIDE      #SET_BIT_10   
PAT #KILLANDSYNC        #KILLANDSYNC
PAT #SHUT_OPEN          #SHUT_OPEN
PAT #SHUT_CLOSE         #SHUT_CLOSE

ORG PAT #MODE_SLCT_SEG                * CELL MODE SELECTION COMMAND WORDS AND MASKS 
PAT #OTA_ROWS       x00000008        * Number of rows to read out in this list. This list is used by READOUT_ALL
PAT #ENABLE_R0      #ENABLE_R0       * First row to read out - Row 0
PAT #ENABLE_R1      #ENABLE_R1       * Second row to read out - Row 1
PAT #ENABLE_R2      #ENABLE_R2
PAT #ENABLE_R3      #ENABLE_R3
PAT #ENABLE_R4      #ENABLE_R4
PAT #ENABLE_R5      #ENABLE_R5
PAT #ENABLE_R6      #ENABLE_R6
PAT #ENABLE_R7      #ENABLE_R7       * Last row to read out - Row 7
PAT #DISABLE_ALL    #DISABLE_ALL
PAT #STANDBY_ALL    #STANDBY_ALL
PAT #ACTIVE_ALL     #ACTIVE_ALL
PAT #ENABLE_GUIDE   #ENABLE_GUIDE
PAT #ACTIVE_GUIDE   #ACTIVE_GUIDE
PAT #PROC_GUIDE     #PROC_GUIDE
PAT #PROC_SCIENCE   #PROC_SCIENCE

ORG PAT #SERIAL_WAVEFORMS             * SERIAL CLOCKING WAVEFORMS HERE
PAT #SER_0          #SER_0
PAT #SER_1          #SER_1
PAT #SER_2          #SER_2
PAT #SER_3          #SER_3
PAT #SER_4          #SER_4
PAT #SER_5          #SER_5
PAT #SER_6          #SER_6
PAT #SER_7          #SER_7

ORG PAT #SHIFT_PATTERNS
PAT #CLEAR_SHIFT    #CLEAR_SHIFT     * Shift pattern used during clear_all routine
PAT #READ_SHIFT     #READ_SHIFT      * Shift charge from parallels into serial
PAT #IDLE_SHIFT	    #IDLE_SHIFT	     * Shift pattern used during idle clearing
PAT #SCI_READ_OVLP  #SCI_READ_OVLP
PAT #GDE_READ_OVLP  #GDE_READ_OVLP
PAT #SHIFT_OVLP	    #SHIFT_OVLP

ORG PAT #CDS_COMMAND_VECTORS
PAT #CDSCMD0        #CDSCMD0
PAT #CDSCMD1        #CDSCMD1

ORG PAT #PATSEG_DR
PAT #DR_ON_LC       x00000008
PAT #DR0_ONCODE     #DR0_ONCODE
PAT #DR1_ONCODE     #DR1_ONCODE
PAT #DR2_ONCODE     #DR2_ONCODE
PAT #DR3_ONCODE     #DR3_ONCODE
PAT #DR4_ONCODE     #DR4_ONCODE
PAT #DR5_ONCODE     #DR5_ONCODE
PAT #DR6_ONCODE     #DR6_ONCODE
PAT #DR7_ONCODE     #DR7_ONCODE
PAT #DR_OFF_LC      x00000008
PAT #DR0_OFFCODE    #DR0_OFFCODE
PAT #DR1_OFFCODE    #DR1_OFFCODE
PAT #DR2_OFFCODE    #DR2_OFFCODE
PAT #DR3_OFFCODE    #DR3_OFFCODE
PAT #DR4_OFFCODE    #DR4_OFFCODE
PAT #DR5_OFFCODE    #DR5_OFFCODE
PAT #DR6_OFFCODE    #DR6_OFFCODE
PAT #DR7_OFFCODE    #DR7_OFFCODE

ORG PAT #CLEAR_WAVEFORMS
PAT #CLEAR_ROWS     #CLEAR_ROWS   
PAT #CLEAR_BINNING  #CLEAR_BINNING
PAT #CLEAR_COLS     #CLEAR_COLS   
PAT #PRESCAN	    #PRESCAN
PAT #CLR_1          #CLR_1
PAT #CLR_2          #CLR_2
PAT #CLR_3          #CLR_3
PAT #CLR_4          #CLR_4
PAT #CLR_5          #CLR_5
PAT #CLR_6          #CLR_6

ORG PAT #GUIDE_WAVEFORMS
PAT #GDE_0          #GDE_0
PAT #GDE_1          #GDE_1
PAT #GDE_2          #GDE_2
PAT #GDE_3          #GDE_3
PAT #GDE_4          #GDE_4
PAT #GDE_5          #GDE_5
PAT #GDE_6          #GDE_6
PAT #GDE_7          #GDE_7
PAT #GCLR_1          #GCLR_1
PAT #GCLR_2          #GCLR_2
PAT #GCLR_3          #GCLR_3
PAT #GCLR_4          #GCLR_4
PAT #GCLR_5          #GCLR_5
PAT #GCLR_6          #GCLR_6

*************************************
* PROGRAM CODE SEGMENT STARTS HERE **
*************************************
*************************************
** EXECUTIVE ROUTINE
*************************************
#MAIN   NOP                           * Bug in the sequencer enable logic dictates that this be the first instruction.
        CAL #INIT                     * Establish initial conditions
#EXEC   LMR #WRITE32
        JCB #CMD_GO #START            * IF START flag set, Go sort out who is bothering us.
        JCB #EX_2 #RUN_EXPOSE         * ELSEIF expose is active, check on the science exposure timer flag
        LPP #STOP_SCI_ITC
        LSR #MCB                      * ELSE cancel the integration timer.
	CAL #IDLE_CLEAR		      * AND clear a few rows of each OTA cell
#EX_1   JCB #EX_3 #RUN_GUIDE          * IF guiding is active, check on the guide exposure timer flag
        LPP #STOP_GD_ITC
        LSR #MCB                      * ELSE cancel the guide timer.
        JMP #EXEC                     * GOTO exec loop
#EX_2   JCB #READ_SCIENCE #SC_ITC     * IF science integration timer has overflowed, readout the science array.
        JMP #EX_1                     * ELSE GOTO EX_1 - check guiding
#EX_3   JCB #READ_GUIDE #GD_ITC       * IF guide integration timer has overflowed, readout the guide cells.
        JMP #EXEC                     * ELSE GOTO exec loop
        
#CMD_GO LPP #KILL_EXPFLG
        LSR #MCB                      * Kill the START flag in the command register
        JCB #START_SHIFT #SHIFT       * IF SHIFT bit set do a shift operation
        JCB #START_EXPOSE #EXPOSE     * ELSEIF exposure bit set do a science exposure
        LPP #RESET_RUN_EXPOSE
        LSR #MCB                      * ELSE cancel the expose run flag.
#CMD_1  JCB #START_GUIDE #GUIDE       * IF guide bit set do a guide exposure
        LPP #RESET_RUN_GUIDE
        LSR #MCB                      * ELSE cancel the Guide exposure run flag.
        JMP #EXEC                     * GOTO exec loop
        
#START_EXPOSE 
        JCB #CMD_1 #RUN_EXPOSE        * Do not re-start an integration that is already running
        LPP #SET_RUN_EXPOSE
        LSR #MCB                      * Set the expose run flag.
        CAL #STNDBY_ALL                 * Put the cell(s) into standby mode 
*        LDA #CLKPORT                    * Point to clock port
*        LPP #SER_1                      * Set up the serials for integrate (S2 high)
*        LSR #CCDCLK
        CAL #OPEN_SHUTTER
        LMR #WRITE32
        LPP #STRT_SCI_ITC
        LSR #MCB                      * Start the science integration timer running.
        LPP #RESET_EXPOSE
        LSR #MCB                      * Reset the command bit
        JMP #CMD_1                    * Go back to see if we need the guide loop running
        
#START_GUIDE 
        JCB #EXEC #RUN_GUIDE          * Do not re-start the guide loop if it is already running
        LPP #SET_RUN_GUIDE
        LSR #MCB                      * Set the guide run flag.
        CAL #DR_ON                    * Leave drain voltages on while guiding
	DUS 100			      * Allow drains to switch on before clearing the guide cells, else we get a band across image
	CAL #GUIDE_CLEAR	      * Clear the guide cells now that the shutter has been open for an indeterminate time - 20071024 DGS
        LPP #STRT_GD_ITC
        LSR #MCB                      * Start the guide integration timer running.
        LPP #RESET_GUIDE
        LSR #MCB                      * Reset the command bit
        JMP #EXEC                     * Go back to drudgery
        
#START_SHIFT
	LMR #WRITE16
	LPP #SHIFT_OVLP			* Set up the parallel clock overlaps for shifting
	LSR #CCDACQ_ALL
	CAL #WAITON_BUSY
        LPP #PROC_SCIENCE
        LSR #CCDACQ_ALL               * so trigger the individual ccd acq boards to process the shift lists
        LMR #WRITE32                  * Put the mode back to normal for our colleagues
        CAL #WAITON_BUSY              * Probably not a good idea :-)
        LPP #RESET_SHIFT
        LSR #MCB                      * Reset the command bit
        JMP #EXEC                     * Go directly back to exec. Illegal to start an exposure or guide loop
                                      * while doing a shift command.                                      
#READ_SCIENCE
        CAL #CLOSE_SHUTTER
        LPP #STOP_SCI_ITC
        LSR #MCB                      * Stop the science integration timer.
        LPP #STOP_GD_ITC
        LSR #MCB                      * Stop the guide integration timer.
        CAL #READOUT_ALL              * Go do a readout of the complete array
        LMR #WRITE32
        LPP #RESET_RUN_GUIDE
        LSR #MCB                      * Cancel the Guide exposure run flag.
        LPP #RESET_RUN_EXPOSE
        LSR #MCB                      * Cancel the expose run flag.
        LPP #RESET_EXPOSE
        LSR #MCB                      * Reset the command bit
        CAL #ACTIVATE_ALL             * Bring the whole array into active mode for IDLE routine
        JMP #EXEC

#READ_GUIDE
        LPP #STOP_GD_ITC
        LSR #MCB                      * Stop the guide integration timer.
        CAL #READOUT_GUIDE
        LMR #WRITE32
        LPP #STRT_GD_ITC
        LSR #MCB                      * Start the guide integration timer running.
        JMP #EXEC

*********************************************
** HOUSEKEEPING ROUTINES
*********************************************

#INIT   				* Initial state
        CAL #CLOSE_SHUTTER
        CAL #DR_ON
        CAL #DR_OFF
        CAL #ACTIVATE_ALL               * Bring the whole array into active mode for IDLE routine
        RET        

#OPEN_SHUTTER
        LMR #WRITE32
        LDA #CLKPORT
        LPP #SHUT_OPEN
        LSR #CCDCLK
        RET

#CLOSE_SHUTTER
        LMR #WRITE32
        LDA #CLKPORT
        LPP #SHUT_CLOSE
        LSR #CCDCLK
        RET

#DR_ON					* Set the Drain Voltage to the ON State
        LMR #WRITE16
        LPP #DR_ON_LC
        LPB
            IPP
            LSR #CCDACQ_ALL
            CAL #WAITON_BUSY 
            DUS 30
        LPE
        LMR #WRITE32
        RET

#DR_OFF					* Set the Drain Voltage to the OFF State
        LMR #WRITE16
        LPP #DR_OFF_LC
        LPB
            IPP
            LSR #CCDACQ_ALL
            CAL #WAITON_BUSY 
            DUS 30
        LPE
        LMR #WRITE32
        RET

#WAITON_BUSY
        JCB #WAITON_BUSY #FUNC_BUSY
        RET

*********************************************
** OTA CELL STATE ROUTINES
*********************************************

#DISABLE_ALL                	* SET ALL CELLS DISABLED VIA CCD DOP
        LMR #WRITE32     
        LPP #DISABLE_ALL
        LDA #MODE_CMD
        LSR #CCDACQ_ALL             
        CAL #WAITON_BUSY 
        RET

#ACTIVATE_ALL             	* SET ALL CELLS ACTIVE VIA CCD DOP
        LMR #WRITE32
        LPP #ACTIVE_ALL
        LDA #MODE_CMD
        LSR #CCDACQ_ALL             
        CAL #WAITON_BUSY 
        RET

#STNDBY_ALL                	* SET ALL CELLS STANDBY VIA CCD DOP
        LMR #WRITE32     
        LPP #STANDBY_ALL
        LDA #MODE_CMD
        LSR #CCDACQ_ALL             
        CAL #WAITON_BUSY
        DUS 50
        RET

*********************************************
** CLEARING ROUTINES
*********************************************

#IDLE_CLEAR        			* Clear a bin of rows and return to EXEC
        LMR #WRITE32   
        LPP #SER_7              	* Set up the serials to dump charge (S1 and S2 high) RG=off
        LDA #CLKPORT            	* Point to clock port
        LSR #CCDCLK
        LPP #IDLE_SHIFT     		* Shift binning value of rows into serials
        LDA #SHIFT_CMD      		* This command done in parallel on all the ccd acquisition cards.
        LSR #CCDACQ_ALL     		* Parallel row is dumped into serial register here for all cells
        CAL #WAITON_BUSY
        LPP #CLEAR_COLS
        LPB                     	* Clear entire serial register
            LDA #CLKPORT
            LPP #CLR_1
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_2
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_3
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_4
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_5
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_6
            LSR #CCDCLK
        LPE                     	* ENDFOR clear cols
        LDA #CLKPORT                    * Point to clock port
        LPP #SER_1                      * Set up the serials for integrate (S2 high)
        LSR #CCDCLK
        RET

#GUIDE_CLEAR                            * Clear all OTA guide cells 
        LMR #WRITE32                    * Set 32 bit write mode
        LPP #ACTIVE_GUIDE
        LDA #MODE_CMD
        LSR #CCDACQ_ALL             	* SET ALL GUIDE CELLS ACTIVE VIA CCD DOP
        CAL #WAITON_BUSY
        LPP #CLEAR_ROWS			* IDLE_SHIFT need to be set so that CLEAR_ROWS x IDLE_SHIFT = 512
        LPB                         	* Loop on number of row shifts for clear operation
            LPP #GCLR_6              	* Set up the serials to dump charge (S1 and S2 high)
            LDA #CLKPORT            	* Point to clock port
            LSR #CCDCLK
            LPP #IDLE_SHIFT     	* Shift binning value of rows into serials
            LDA #SHIFT_CMD      	* This command done in parallel on all the ccd acquisition cards.
            LSR #CCDACQ_ALL     	* Parallel row is dumped into serial register here for all cells
            CAL #WAITON_BUSY
            CAL #SER_GCLEAR          	* Clear the serial register
          LPE                           * ENDFOR clear rows
        CAL #STNDBY_ALL                 * Put the cell(s) into standby ready for next row
        LPP #GDE_1                      * Set up the serials for integrate (S2 high) RG=off
        LDA #CLKPORT                    * Point to clock port
        LSR #CCDCLK
        RET

#SER_GCLEAR 				*Routine to clear serial register during guiding (i.e. shutter open)
        LPP #CLEAR_COLS
        LPB                             * Clear the entire serial register holding the shutter open
            LDA #CLKPORT
            LPP #GCLR_1
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #GCLR_2
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #GCLR_3
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #GCLR_4
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #GCLR_5
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #GCLR_6
            LSR #CCDCLK
        LPE                             * ENDFOR cols
        RET

#SER_CLEAR 				*Routine to clear serial register during guiding (i.e. shutter open)
        LPP #CLEAR_COLS
        LPB                             * Clear the entire serial register holding the shutter open
            LDA #CLKPORT
            LPP #CLR_1
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_2
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_3
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_4
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_5
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_6
            LSR #CCDCLK
        LPE                             * ENDFOR cols
        RET

#ROW_DUMP        			* Clear a binned row 
        LMR #WRITE32   
*	CAL #ACTIVATE_ALL		* Enable all cells for dumping rows
        LPP #SER_7              	* Set up the serials to dump charge (S1 and S2 high) RG=off
        LDA #CLKPORT            	* Point to clock port
        LSR #CCDCLK
        LRB #ROWBIN    		* Transfer rowbin rows into serial register      
            LPP #READ_SHIFT         * x0000FFFF is -1 vertical shift to transfer charge to serial register
            LDA #SHIFT_CMD
            LSR #CCDACQ_ALL         * Shift the charge down one row
            CAL #WAITON_BUSY 
        LPE				* End row binning
        LPP #CLEAR_COLS
        LPB                     	* Clear entire serial register
            LDA #CLKPORT
            LPP #CLR_1
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_2
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_3
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_4
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_5
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #CLR_6
            LSR #CCDCLK
        LPE                     	* ENDFOR clear cols
*	CAL #STNDBY_ALL			* Put all cells into standby mode
        RET

*********************************************
** SCIENCE READOUT ROUTINES
*********************************************

#READOUT_ALL
	LMR #WRITE16
	LPP #SCI_READ_OVLP		* Set up the parallel clock overlaps for science read
	LSR #CCDACQ_ALL
	CAL #WAITON_BUSY
	LMR #WRITE32
        CAL #DR_ON			* Turn on the output drains
        CAL #STNDBY_ALL         	* Start with a fresh array mode
        LPP #OTA_ROWS           	* Point to the start of the row read list
        LPB                     	* Loop on number of ota rows which is first item in list
            IPP                 	* Point to next row enable mask that selects the OTA row
            CAL #ROW_WRAPPER    	* For read out
        LPE                     	* End of ota row loop
        CAL #DR_OFF			* Turn off the output drains
        RET

#ROW_WRAPPER
        LDA #MODE_CMD
        LSR #CCDACQ_ALL             	* SET SELECTED ROW TO ENABLED
        CAL #WAITON_BUSY            	* WAIT FOR THE ENABLE FUNCTION TO FINISH
*	CAL #ROW_DUMP			* Dumps a binned row into the serial register to eliminate bright rows
        JCB #READ_ROW #SBIN_N		* Jump to read row (unbinned) if not binning flag is set
        JMP #READ_BINNED_ROW		* Else jump to read row (binned) 
#RD_DONE CAL #STNDBY_ALL             	* Put the cell(s) into standby ready for next row
        RET

#READ_ROW  				* Read a row of cells
        LDA #CLKPORT
        LPP #SER_7      		* Set up the serials to dump charge (S1 and S2 high)
        LSR #CCDCLK
        LRB #ROWCNT             	* FOR number of unbinned rows (rows in a cell not OTA rows)
            LPP #READ_SHIFT         * x0000FFFF is -1 vertical shift to transfer charge to serial register
            LDA #SHIFT_CMD
            LSR #CCDACQ_ALL         * Shift the charge down one row
            CAL #WAITON_BUSY 
            LPP #PRESCAN		* Skip the serials ahead by the number of prescan pixels
            LPB                         * Loop on number of prescan pixels
		CAL #S_SKIP
	    LPE
	    LRB #COLCNT			* For the total number of unbinned columns
	        LPP #SER_0		* Fire the reset gate
            	LSR #CCDCLK
            	DSC #RST_WIDTH          * Reset hold time
		CAL #S_CTC		* Convert
	    LPE				* End column binning
	LPE				* End row reads
	JMP #RD_DONE

#READ_BINNED_ROW  				* Read a row of cells
        LDA #CLKPORT
        LPP #SER_7      		* Set up the serials to dump charge (S1 and S2 high)
        LSR #CCDCLK
        LRB #ROWCNT             	* FOR number of binned rows (rows in a cell not OTA rows)
            LRB #ROWBIN    		* Transfer rowbin rows into serial register      
                LPP #READ_SHIFT         * x0000FFFF is -1 vertical shift to transfer charge to serial register
                LDA #SHIFT_CMD
                LSR #CCDACQ_ALL         * Shift the charge down one row
                CAL #WAITON_BUSY 
            LPE				* End row binning
            LPP #PRESCAN		* Skip the serials ahead by the number of prescan pixels
            LPB                         * Loop on number of prescan pixels
		CAL #S_SKIP
	    LPE
	    LRB #COLCNT			* For the total number of binned columns
	        LPP #SER_0		* Fire the reset gate
            	LSR #CCDCLK
            	DSC #RST_WIDTH          * Reset hold time
		LRB #COLBIN		* For the number of columns to bin (minus one)
	    	    CAL #S_SKIP		* Else bin the columns
		LPE
		CAL #S_CTC		* Convert
	    LPE				* End column binning
	LPE				* End row reads
	JMP #RD_DONE

#S_CTC        				* Clock the serials for conversion
        LPP #CDSCMD0            	* Load the CDS microsequencer trigger word
        LDA #SEQ_CMD
        LSR #CCDACQ_ALL         	* Fire the CDS microsequencer
        LDA #CLKPORT
        LPP #SER_1
        LSR #CCDCLK
        DSC #RST_SETTLE         	* Recovery from reset
        LPP #SER_2
        LSR #CCDCLK
        DSC #SER_OVERLAP
        LPP #SER_3
        LSR #CCDCLK
        DSC #SER_OVERLAP
        LPP #SER_4
        LSR #CCDCLK
        DSC #SER_OVERLAP
        LPP #SER_5
        LSR #CCDCLK
        DSC #SCI_DWELL			* Hold on while the CDS integrates on RST
        LPP #SER_6			* Now dump the Video
        LSR #CCDCLK
        DSC #SCI_DWELL			* Hold on while the CDS integrates on VID
        LPP #SER_7			* Now back to PXFER state
        LSR #CCDCLK
        DSC 10
        RET

#S_SKIP        				* Clock the serials for conversion
        LDA #CLKPORT
        LPP #SER_1
        LSR #CCDCLK
        DSC #SER_OVERLAP
        LPP #SER_2
        LSR #CCDCLK
        DSC #SER_OVERLAP
        LPP #SER_3
        LSR #CCDCLK
        DSC #SER_OVERLAP
        LPP #SER_4
        LSR #CCDCLK
        DSC #SER_OVERLAP
        LPP #SER_6	
        LSR #CCDCLK
        DSC #SER_OVERLAP
        LPP #SER_7
        LSR #CCDCLK
        DSC #SER_OVERLAP
	RET

*********************************************
** GUIDE READOUT ROUTINES
*********************************************

#READOUT_GUIDE
	LMR #WRITE16
	LPP #GDE_READ_OVLP		* Set up the parallel clock overlaps for guide read
	LSR #CCDACQ_ALL
	CAL #WAITON_BUSY
        LMR #WRITE32
        LDA #CLKPORT            	* Point to clock port
        LPP #GDE_7      		* Set up the serials to dump charge (S1 and S2 high)
        LSR #CCDCLK
        LMR #WRITE16                	* Shift the guide regions to the origin of the cell
        LPP #PROC_GUIDE
        LSR #CCDACQ_ALL             	* We expect that the shift list structures have been previously downloaded
        LMR #WRITE32                	* Put the mode back to normal for our colleagues                                      
        CAL #WAITON_BUSY
	CAL #SER_GCLEAR			* Clear out the serial register
        LPP #ENABLE_GUIDE
        LDA #MODE_CMD
        LSR #CCDACQ_ALL             	* SET ALL GUIDE CELLS ACTIVE VIA CCD DOP
        CAL #WAITON_BUSY
	JCB #READ_GDEROW #GBIN_N	* Jump to read row (unbinned) if not binning flag set
	JMP #READ_BINNED_GDEROW		* Else jump to read row (binned)
#GRD_DONE CAL #STNDBY_ALL		* Put the cells back into standby mode
*        LMR #WRITE16                  	* We expect that the shift list structures have been previously downloaded
*        LPP #PROC_SCIENCE
*        LSR #CCDACQ_ALL               	* so trigger the individual ccd acq boards to process the shift lists
*        LMR #WRITE32                  	* Put the mode back to normal for our colleagues
        RET

#READ_GDEROW
        LRB #GDECNT             	* FOR number of unbinned guide rows
	    CAL #BACK_WASH		* Clock the serials backwards to clear for guide box for the next rows
            LDA #CLKPORT            	* Point to clock port
            LPP #GDE_7      		* Set up the serials to dump charge (S1 and S2 high)
            LSR #CCDCLK
            LPP #READ_SHIFT         	* x0000FFFF is -1 vertical shift to transfer charge to serial register
            LDA #SHIFT_CMD
            LSR #CCDACQ_ALL         	* Shift the charge down one row
            CAL #WAITON_BUSY 
            LPP #PRESCAN		* Skip the serials ahead by the number of prescan pixels
            LPB                         * Loop on number of prescan pixels
		CAL #G_SKIP
	    LPE
            LDA #CLKPORT
	    LRB #GDECNT			* For the total number of binned guide columns
                LPP #GDE_0		* Fire the reset gate
            	LSR #CCDCLK
            	DSC #RST_WIDTH          * Reset hold time
	    	CAL #G_CTC		* Else bin the columns
	    LPE				* End column binning
	LPE				* End row reads
	JMP #GRD_DONE

#READ_BINNED_GDEROW
        LRB #GDECNT             	* FOR number of binned guide rows
	    CAL #BACK_WASH		* Clock the serials backwards to clear for guide box for the next rows
            LDA #CLKPORT            	* Point to clock port
            LPP #GDE_7      		* Set up the serials to dump charge (S1 and S2 high)
            LSR #CCDCLK
            LRB #GDEROWBIN    		* Transfer GdeRowBin rows into serial register      
                LPP #READ_SHIFT         * x0000FFFF is -1 vertical shift to transfer charge to serial register
                LDA #SHIFT_CMD
                LSR #CCDACQ_ALL         * Shift the charge down one row
                CAL #WAITON_BUSY 
            LPE				* End row binning
            LPP #PRESCAN		* Skip the serials ahead by the number of prescan pixels
            LPB                         * Loop on number of prescan pixels
		CAL #G_SKIP
	    LPE
	    LRB #GDECNT			* For the total number of binned guide columns
                LPP #GDE_0		* Fire the reset gate
            	LSR #CCDCLK
            	DSC #RST_WIDTH          * Reset hold time
		LRB #GDECOLBIN		* For the number of columns to bin (minus one)
	    	    CAL #G_SKIP		* Else bin the columns
		LPE
	    	CAL #G_CTC		* Convert
	    LPE				* End column binning
	LPE				* End row reads
	JMP #GRD_DONE

#G_CTC        				* Clock the serials for conversion
        LPP #CDSCMD1            	* Load the CDS microsequencer trigger word
        LDA #SEQ_CMD
        LSR #CCDACQ_ALL         	* Fire the CDS microsequencer
        LDA #CLKPORT
        LPP #GDE_1
        LSR #CCDCLK
        DSC #RST_SETTLE         	* Recovery from reset
        LPP #GDE_2
        LSR #CCDCLK
        DSC #GDE_OVERLAP
        LPP #GDE_3
        LSR #CCDCLK
        DSC #GDE_OVERLAP
        LPP #GDE_4
        LSR #CCDCLK
        DSC #GDE_OVERLAP
        LPP #GDE_5
        LSR #CCDCLK
        DSC #GDE_DWELL			* Hold on while the CDS integrates on RST
        LPP #GDE_6			* Now dump the Video
        LSR #CCDCLK
        DSC #GDE_DWELL			* Hold on while the CDS integrates on VID
        LPP #GDE_7			* Now back to PXFER state
        LSR #CCDCLK
        DSC 10
        RET

#G_SKIP
        LDA #CLKPORT
        LPP #GDE_1
        LSR #CCDCLK
        DSC #GDE_OVERLAP
        LPP #GDE_2
        LSR #CCDCLK
        DSC #GDE_OVERLAP
        LPP #GDE_3
        LSR #CCDCLK
        DSC #GDE_OVERLAP
        LPP #GDE_4
        LSR #CCDCLK
        DSC #GDE_OVERLAP
        LPP #GDE_6
        LSR #CCDCLK
        DSC #GDE_OVERLAP
        LPP #GDE_7
        LSR #CCDCLK
        DSC #GDE_OVERLAP
	RET

#BACK_WASH	    			* Back up the serial register by guide box size (clear) RG=on
        LDA #CLKPORT
	LRB #GDEROWBIN			* For the guide binning factor
	    LRB #GDECNT 		* Times the binned guide size (gives actual pixel count)
	        LPP #GCLR_6
             	LSR #CCDCLK
                DSC #CLR_OVERLAP
                LPP #GCLR_5
                LSR #CCDCLK
                DSC #CLR_OVERLAP
                LPP #GCLR_4
                LSR #CCDCLK
                DSC #CLR_OVERLAP
                LPP #GCLR_3
                LSR #CCDCLK
                DSC #CLR_OVERLAP
                LPP #GCLR_2
                LSR #CCDCLK
                DSC #CLR_OVERLAP
                LPP #GCLR_1
                LSR #CCDCLK
                DSC #CLR_OVERLAP
	    LPE				* End GdeSize loop
	LPE				* End GdeRowBin loop
        LPP #PRESCAN		* Back up the prescan pixels to clear them too
        LPB                         * Loop on number of prescan pixels
	    LPP #GCLR_6
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #GCLR_5
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #GCLR_4
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #GCLR_3
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #GCLR_2
            LSR #CCDCLK
            DSC #CLR_OVERLAP
            LPP #GCLR_1
            LSR #CCDCLK
            DSC #CLR_OVERLAP
	LPE
	RET

**************************************************
** End of sequencer code segment

